==> spectrojack-0.3.tar.gz: spectrojack-0.3/colors.c <==
#include "colors.h"
#include "defs.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

typedef struct {
  unsigned char *r;
  unsigned char *g;
  unsigned char *b;
  float *v;
  int size;
  int maxsize;
} color_table;

static void add_color(color_table *t, float pos, int r, int g, int b)
{
  int i;
  if (t->size == t->maxsize) {
    t->maxsize += 512;
    if (!t->size) {
      t->r = malloc(t->maxsize);
      t->g = malloc(t->maxsize);
      t->b = malloc(t->maxsize);
      t->v = malloc(t->maxsize * sizeof(float));
    } else {
      t->r = realloc(t->r, t->maxsize);
      t->g = realloc(t->g, t->maxsize);
      t->b = realloc(t->b, t->maxsize);
      t->v = realloc(t->v, t->maxsize * sizeof(float));
    }
    if (!t->r || !t->g || !t->b || !t->v) { printf("oom\n"); exit(1); }
  }
  t->r[t->size] = r;
  t->g[t->size] = g;
  t->b[t->size] = b;
  t->v[t->size] = pos;
  i = t->size;
  t->size++;
  while (i) {
    unsigned char v;
    float f;
    if (t->v[i-1] < t->v[i]) break;
    f = t->v[i]; t->v[i] = t->v[i-1]; t->v[i-1] = f;
    v = t->r[i]; t->r[i] = t->r[i-1]; t->r[i-1] = v;
    v = t->g[i]; t->g[i] = t->g[i-1]; t->g[i-1] = v;
    v = t->b[i]; t->b[i] = t->b[i-1]; t->b[i-1] = v;
    i--;
  }
}

static void free_color_table(color_table *t)
{
  if (t->v) free(t->v);
  if (t->r) free(t->r);
  if (t->g) free(t->g);
  if (t->b) free(t->b);
}

static float color_table_pos(color_table *t, float p)
{
  int i;
  /* TODO: speed this up (dichotomy or something) */
  /* well, this is not critical either, so no need to bother... */
  if (p >= t->v[t->size-1]) return t->size-1;
  for (i = 0; i < t->size-1; i++) if (t->v[i] <= p && p < t->v[i+1]) break;
  if (i == t->size-1) abort();
  return R3(p, t->v[i], t->v[i+1], i, i+1);
}

static int buildmap(colormap *m, color_table *t, int v)
{
  float p0, p1, p, il;
  int i;
  int im;
  if (t->size < 2)
    { if (v) printf("colormap too small, can't build\n"); return 0; }
  if (t->v[0] >= t->v[1])
    { if (v) printf("colormap not correct\n"); return 0; }
  p0 = t->v[0];
  p1 = t->v[t->size-1];
  /* double end color for no problem with im+1 index below */
  add_color(t, t->v[t->size-1],
            t->r[t->size-1], t->g[t->size-1], t->b[t->size-1]);
  /* and cheat size */
  t->size--;
  for (i = 0; i < m->len; i++) {
    p = R3(i, 0, m->len-1, p0, p1);
    il = color_table_pos(t, p);
    im = floor(il);
    m->rgb[i*3] = R3(il, im, im+1, t->r[im], t->r[im+1]);
    m->rgb[i*3+1] = R3(il, im, im+1, t->g[im], t->g[im+1]);
    m->rgb[i*3+2] = R3(il, im, im+1, t->b[im], t->b[im+1]);
  }
  return 1;
}

int load_colormap(colormap *m, const char *filename, int v)
{
  int line = 0;
  int d0, d1, d2;
  float f0, f1, f2, f3;
  char l[1024];
  FILE *f = 0;
  int p;
  color_table t;

  memset(&t, 0, sizeof(color_table));
  m->len = 256;
  m->rgb = malloc(256*3); if (!m->rgb) { if (v) printf("oom\n"); goto err; }
  memset(m->rgb, 0, 256*3);
  m->mindB = -120;
  m->maxdB = 0;

  f = fopen(filename, "rb");
  if (!f) { if (v) printf("can't load colormap %s\n", filename); goto err; }
  while (!feof(f)) {
    p = 0;
    while(p < 1024) { int c=fgetc(f); if (c=='\n'||c==EOF) break; l[p++]=c; }
    line++;
    if (p == 1024) { if (v) printf("line too long\n"); goto err; }
    if (p == 0) break;
    l[p] = 0;
    if (sscanf(l, "mapsize %d", &d0) == 1) {
      if (d0 < 2 || d0 > 1000000)
        {if (v) printf("wrong mapsize %d\n",d0); goto err;}
      free(m->rgb);
      m->rgb = malloc(d0*3);
      if (!m->rgb) { if (v) printf("oom\n"); line=0; goto err; }
      memset(m->rgb, 0, d0*3);
      m->len = d0;
    } else if (sscanf(l, "color %g %d %d %d", &f0, &d0, &d1, &d2) == 4) {
      if (f0 < 0 || f0 > 1)
      { if (v) printf("bad color range (%g), must be in [0.0-1.0]\n", f0);
        goto err; }
      if (d0 < 0 || d0 > 255 || d1 < 0 || d1 > 255 || d2 < 0 || d2 > 255)
      { if (v) printf("bad color, RGB must be in [0-255], integer\n");
        goto err; }
      add_color(&t, f0, d0, d1, d2);
    } else if (sscanf(l, "colorb %g %g %g %g", &f0, &f1, &f2, &f3) == 4) {
      if (f0 < 0 || f0 > 1)
      { if (v) printf("bad color range (%g), must be in [0.0-1.0]\n", f0);
        goto err; }
      if (f1 < 0 || f1 > 1 || f2 < 0 || f2 > 1 || f3 < 0 || f3 > 1)
      { if (v) printf("bad color, RGB must be in [0.0-1.0]\n"); goto err; }
      add_color(&t, f0,
          R3(f1, 0, 1, 0, 255), R3(f2, 0, 1, 0, 255), R3(f3, 0, 1, 0, 255));
    } else if (sscanf(l, "cell %d %g %g %g", &d0, &f0, &f1, &f2) == 4) {
      if (d0 < 0 || d0 >= m->len)
      { if (v) printf("bad cell range (%d), must be in [0-%d]\n", d0, m->len);
        goto err; }
      if (f0 < 0 || f0 > 1 || f1 < 0 || f1 > 1 || f2 < 0 || f2 > 1)
      { if (v) printf("bad color, RGB must be in [0.0-1.0]\n"); goto err; }
      m->rgb[d0*3] = R3(f0, 0, 1, 0, 255);
      m->rgb[d0*3+1] = R3(f1, 0, 1, 0, 255);
      m->rgb[d0*3+2] = R3(f2, 0, 1, 0, 255);
    } else if (!strcmp(l, "buildmap")) {
      if (!buildmap(m, &t, v))
        { if (v) printf("can't build colormap\n"); goto err; }
    } else if (sscanf(l, "dBmin %g", &f0) == 1) {
      m->mindB = f0;
    } else if (sscanf(l, "dBmax %g", &f0) == 1) {
      m->maxdB = f0;
    } else {
      if (v) printf("%s: unknown (or bad) command '%s'\n", filename, l);
      goto err;
    }
  }
  if (line == 1 && p == 0) { line = 0; goto err; }
  fclose(f);
  free_color_table(&t);
  if (m->maxdB<m->mindB+0.1)
    { if (v) printf("bad mindB/maxdB\n"); line=0; goto err; }
  return 0;
err:
  if (line) if(v) printf("%s: error line %d\n", filename, line);
  free_color_table(&t);
  if (f) fclose(f);
  if (m->rgb) free(m->rgb);
  return -1;
}

unsigned char *get_color(colormap *m, float dB)
{
  int pos;
  if (dB < m->mindB) dB = m->mindB;
  if (dB > m->maxdB) dB = m->maxdB;
  pos = R3(dB, m->mindB, m->maxdB, 0, m->len);
  /* TODO: remove this if by duplicating last color of colormap */
  if (pos > m->len - 1) pos = m->len - 1;
  return m->rgb + 3 * pos;
}

int default_colormap(colormap *m)
{
  int i;
  if (!load_colormap(m, DEF_COLORMAP_DIR "/default.txt", 0)) return 0;
  /* can't load default, revert to black and white */
  m->rgb = malloc(256*3); if (!m->rgb) { printf("oom\n"); return -1; }
  m->len = 256;
  m->mindB = -120;
  m->maxdB = 0;
  for (i = 0; i < 256; i++) m->rgb[i*3] = m->rgb[i*3+1] = m->rgb[i*3+2] = i;
  return 0;
}

void free_colormap(colormap *m)
{
  free(m->rgb);
}

==> spectrojack-0.4.tar.gz: spectrojack-0.4/colors.c <==
#include "colors.h"
#include "defs.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

typedef struct {
  unsigned char *r;
  unsigned char *g;
  unsigned char *b;
  float *v;
  int size;
  int maxsize;
} color_table;

static void add_color(color_table *t, float pos, int r, int g, int b)
{
  int i;
  if (t->size == t->maxsize) {
    t->maxsize += 512;
    if (!t->size) {
      t->r = malloc(t->maxsize);
      t->g = malloc(t->maxsize);
      t->b = malloc(t->maxsize);
      t->v = malloc(t->maxsize * sizeof(float));
    } else {
      t->r = realloc(t->r, t->maxsize);
      t->g = realloc(t->g, t->maxsize);
      t->b = realloc(t->b, t->maxsize);
      t->v = realloc(t->v, t->maxsize * sizeof(float));
    }
    if (!t->r || !t->g || !t->b || !t->v) { printf("oom\n"); exit(1); }
  }
  t->r[t->size] = r;
  t->g[t->size] = g;
  t->b[t->size] = b;
  t->v[t->size] = pos;
  i = t->size;
  t->size++;
  while (i) {
    unsigned char v;
    float f;
    if (t->v[i-1] < t->v[i]) break;
    f = t->v[i]; t->v[i] = t->v[i-1]; t->v[i-1] = f;
    v = t->r[i]; t->r[i] = t->r[i-1]; t->r[i-1] = v;
    v = t->g[i]; t->g[i] = t->g[i-1]; t->g[i-1] = v;
    v = t->b[i]; t->b[i] = t->b[i-1]; t->b[i-1] = v;
    i--;
  }
}

static void free_color_table(color_table *t)
{
  if (t->v) free(t->v);
  if (t->r) free(t->r);
  if (t->g) free(t->g);
  if (t->b) free(t->b);
}

static float color_table_pos(color_table *t, float p)
{
  int i;
  /* TODO: speed this up (dichotomy or something) */
  /* well, this is not critical either, so no need to bother... */
  if (p >= t->v[t->size-1]) return t->size-1;
  for (i = 0; i < t->size-1; i++) if (t->v[i] <= p && p < t->v[i+1]) break;
  if (i == t->size-1) abort();
  return R3(p, t->v[i], t->v[i+1], i, i+1);
}

static int buildmap(colormap *m, color_table *t, int v)
{
  float p0, p1, p, il;
  int i;
  int im;
  if (t->size < 2)
    { if (v) printf("colormap too small, can't build\n"); return 0; }
  if (t->v[0] >= t->v[1])
    { if (v) printf("colormap not correct\n"); return 0; }
  p0 = t->v[0];
  p1 = t->v[t->size-1];
  /* double end color for no problem with im+1 index below */
  add_color(t, t->v[t->size-1],
            t->r[t->size-1], t->g[t->size-1], t->b[t->size-1]);
  /* and cheat size */
  t->size--;
  for (i = 0; i < m->len; i++) {
    p = R3(i, 0, m->len-1, p0, p1);
    il = color_table_pos(t, p);
    im = floor(il);
    m->rgb[i*3] = R3(il, im, im+1, t->r[im], t->r[im+1]);
    m->rgb[i*3+1] = R3(il, im, im+1, t->g[im], t->g[im+1]);
    m->rgb[i*3+2] = R3(il, im, im+1, t->b[im], t->b[im+1]);
  }
  return 1;
}

int load_colormap(colormap *m, const char *filename, int v)
{
  int line = 0;
  int d0, d1, d2;
  float f0, f1, f2, f3;
  char l[1024];
  FILE *f = 0;
  int p = 0;
  color_table t;

  memset(&t, 0, sizeof(color_table));
  m->len = 256;
  m->rgb = malloc(256*3); if (!m->rgb) { if (v) printf("oom\n"); goto err; }
  memset(m->rgb, 0, 256*3);
  m->mindB = -120;
  m->maxdB = 0;

  f = fopen(filename, "rb");
  if (!f) { if (v) printf("can't load colormap '%s'\n", filename); goto err; }
  while (!feof(f)) {
    p = 0;
    while(p < 1024) { int c=fgetc(f); if (c=='\n'||c==EOF) break; l[p++]=c; }
    line++;
    if (p == 1024) { if (v) printf("line too long\n"); goto err; }
    if (p == 0) break;
    l[p] = 0;
    if (sscanf(l, "mapsize %d", &d0) == 1) {
      if (d0 < 2 || d0 > 1000000)
        {if (v) printf("wrong mapsize %d\n",d0); goto err;}
      free(m->rgb);
      m->rgb = malloc(d0*3);
      if (!m->rgb) { if (v) printf("oom\n"); line=0; goto err; }
      memset(m->rgb, 0, d0*3);
      m->len = d0;
    } else if (sscanf(l, "color %g %d %d %d", &f0, &d0, &d1, &d2) == 4) {
      if (f0 < 0 || f0 > 1)
      { if (v) printf("bad color range (%g), must be in [0.0-1.0]\n", f0);
        goto err; }
      if (d0 < 0 || d0 > 255 || d1 < 0 || d1 > 255 || d2 < 0 || d2 > 255)
      { if (v) printf("bad color, RGB must be in [0-255], integer\n");
        goto err; }
      add_color(&t, f0, d0, d1, d2);
    } else if (sscanf(l, "colorb %g %g %g %g", &f0, &f1, &f2, &f3) == 4) {
      if (f0 < 0 || f0 > 1)
      { if (v) printf("bad color range (%g), must be in [0.0-1.0]\n", f0);
        goto err; }
      if (f1 < 0 || f1 > 1 || f2 < 0 || f2 > 1 || f3 < 0 || f3 > 1)
      { if (v) printf("bad color, RGB must be in [0.0-1.0]\n"); goto err; }
      add_color(&t, f0,
          R3(f1, 0, 1, 0, 255), R3(f2, 0, 1, 0, 255), R3(f3, 0, 1, 0, 255));
    } else if (sscanf(l, "cell %d %g %g %g", &d0, &f0, &f1, &f2) == 4) {
      if (d0 < 0 || d0 >= m->len)
      { if (v) printf("bad cell range (%d), must be in [0-%d]\n", d0, m->len);
        goto err; }
      if (f0 < 0 || f0 > 1 || f1 < 0 || f1 > 1 || f2 < 0 || f2 > 1)
      { if (v) printf("bad color, RGB must be in [0.0-1.0]\n"); goto err; }
      m->rgb[d0*3] = R3(f0, 0, 1, 0, 255);
      m->rgb[d0*3+1] = R3(f1, 0, 1, 0, 255);
      m->rgb[d0*3+2] = R3(f2, 0, 1, 0, 255);
    } else if (!strcmp(l, "buildmap")) {
      if (!buildmap(m, &t, v))
        { if (v) printf("can't build colormap\n"); goto err; }
    } else if (sscanf(l, "dBmin %g", &f0) == 1) {
      m->mindB = f0;
    } else if (sscanf(l, "dBmax %g", &f0) == 1) {
      m->maxdB = f0;
    } else {
      if (v) printf("%s: unknown (or bad) command '%s'\n", filename, l);
      goto err;
    }
  }
  if (line == 1 && p == 0) { line = 0; goto err; }
  fclose(f);
  free_color_table(&t);
  if (m->maxdB<m->mindB+0.1)
    { if (v) printf("bad mindB/maxdB\n"); line=0; goto err; }
  return 0;
err:
  if (line) if(v) printf("%s: error line %d\n", filename, line);
  free_color_table(&t);
  if (f) fclose(f);
  if (m->rgb) free(m->rgb);
  return -1;
}

unsigned char *get_color(colormap *m, float dB)
{
  int pos;
  if (dB < m->mindB) dB = m->mindB;
  if (dB > m->maxdB) dB = m->maxdB;
  pos = R3(dB, m->mindB, m->maxdB, 0, m->len);
  /* TODO: remove this if by duplicating last color of colormap */
  if (pos > m->len - 1) pos = m->len - 1;
  return m->rgb + 3 * pos;
}

int default_colormap(colormap *m)
{
  int i;
  if (!load_colormap(m, DEF_COLORMAP_DIR "/default.txt", 0)) return 0;
  /* can't load default, revert to black and white */
  m->rgb = malloc(256*3); if (!m->rgb) { printf("oom\n"); return -1; }
  m->len = 256;
  m->mindB = -120;
  m->maxdB = 0;
  for (i = 0; i < 256; i++) m->rgb[i*3] = m->rgb[i*3+1] = m->rgb[i*3+2] = i;
  return 0;
}

void free_colormap(colormap *m)
{
  free(m->rgb);
}
